@@ -54,33 +54,34 @@
 #'       \code{prediction} \tab One column with the predicted class (factor) for classification and the predicted numerical value for regression. One probability per class for probability estimation in several columns. Nothing for survival, refer to \code{object$forest$chf} for the CHF node predictions. \cr
 #'   }
 #' @examples
-#' require(ranger)
 #' rf <- ranger(Species ~ ., data = iris)
 #' treeInfo(rf, 1)
 #' @seealso \code{\link{ranger}}
 #' @author Marvin N. Wright
 #' @export
 treeInfo <- function(object, tree = 1) {
-  if (class(object) != "ranger" & class(object) != "holdoutRF") {
+  if (!inherits(object, "ranger")) {
     stop("Error: Invalid class of input object.")
-  } 
+  }
   forest <- object$forest
   if (is.null(forest)) {
     stop("Error: No saved forest in ranger object. Please set write.forest to TRUE when calling ranger.")
   }
-  if (is.null(forest$dependent.varID) || is.null(forest$num.trees) ||
+  if (is.null(forest$num.trees) ||
       is.null(forest$child.nodeIDs) || is.null(forest$split.varIDs) ||
       is.null(forest$split.values) || is.null(forest$independent.variable.names) ||
       is.null(forest$treetype)) {
     stop("Error: Invalid forest object.")
   }
-  if (forest$treetype == "Survival" && (is.null(forest$status.varID)  ||
-                                        is.null(forest$chf) || is.null(forest$unique.death.times))) {
+  if (forest$treetype == "Survival" && (is.null(forest$chf) || is.null(forest$unique.death.times))) {
     stop("Error: Invalid forest object.")
   }
   if (length(forest$child.nodeIDs) != forest$num.trees || length(forest$child.nodeIDs[[1]]) != 2) {
     stop("Error: Invalid forest object. Is the forest grown in ranger version <0.3.9? Try with the same version the forest was grown.")
   }
+  if (!is.null(forest$dependent.varID)) {
+    forest <- convert.pre.xy(forest, trees = tree)
+  }
   if (tree > forest$num.trees) {
     stop("Error: Requesting tree ", tree, ", but forest has only ", forest$num.trees, " trees.")
   }
@@ -99,25 +100,19 @@ treeInfo <- function(object, tree = 1) {
   result$splitvarID[result$terminal] <- NA
   result$splitvarName[result$terminal] <- NA
   result$splitval[result$terminal] <- NA
-
-  ## Get names of splitting variables 
-  # should be -1 for all >= dependent.varID but +1 change for 1-index
-  # for survival another -1 if >= status.varID
-  independent.varID <- result$splitvarID
-  idx <- !is.na(result$splitvarID) & result$splitvarID < forest$dependent.varID
-  independent.varID[idx] <- result$splitvarID[idx] + 1
-  if (forest$treetype == "Survival") {
-    idx <- !is.na(result$splitvarID) & result$splitvarID >= forest$status.varID
-    independent.varID[idx] <- independent.varID[idx] - 1
-  }
-  result$splitvarName <- forest$independent.variable.names[independent.varID]
+  result$splitvarName <- forest$independent.variable.names[result$splitvarID + 1]
 
   ## Unordered splitting
   idx.unordered <- !result$terminal & !forest$is.ordered[result$splitvarID + 1]
   if (any(idx.unordered)) {
-    result$splitval[idx.unordered] <- sapply(result$splitval[idx.unordered], function(x) {
-      paste(which(as.logical(intToBits(x))), collapse = ",")
-    })
+    if (any(result$splitval[idx.unordered] > (2^31 - 1))) {
+      warning("Unordered splitting levels can only be shown for up to 31 levels.")
+      result$splitval[idx.unordered] <- NA
+    } else {
+      result$splitval[idx.unordered] <- sapply(result$splitval[idx.unordered], function(x) {
+        paste(which(as.logical(intToBits(x))), collapse = ",")
+      })
+    }
   }
   
   ## Prediction
@@ -125,15 +120,17 @@ treeInfo <- function(object, tree = 1) {
     result$prediction <- forest$split.values[[tree]]
     result$prediction[!result$terminal] <- NA
     if (!is.null(forest$levels)) {
-      result$prediction <- factor(result$prediction, levels = forest$class.values, labels = forest$levels)
+      result$prediction <- integer.to.factor(result$prediction, labels = forest$levels)
     }
   } else if (forest$treetype == "Regression") {
     result$prediction <- forest$split.values[[tree]]
     result$prediction[!result$terminal] <- NA
   } else if (forest$treetype == "Probability estimation") {
-    predictions <- matrix(nrow = nrow(result), ncol = length(forest$levels))
+    predictions <- matrix(nrow = nrow(result), ncol = length(forest$class.values))
     predictions[result$terminal, ] <- do.call(rbind, forest$terminal.class.counts[[tree]])
-    colnames(predictions) <- paste0("pred.", forest$levels)
+    colnames(predictions) <- forest$levels[forest$class.values]
+    predictions <- predictions[, forest$levels[sort(forest$class.values)], drop = FALSE]
+    colnames(predictions) <- paste0("pred.", colnames(predictions))
     result <- data.frame(result, predictions)
   } else if (forest$treetype == "Survival") {
     # No prediction for survival (CHF too large?)
