@@ -17,8 +17,8 @@
 namespace ranger {
 
 Tree::Tree() :
-    mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), deterministic_varIDs(0), split_select_varIDs(0), split_select_weights(
-        0), case_weights(0), manual_inbag(0), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), regularization_factor(0), regularization_usedepth(
+    mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), min_bucket(0), deterministic_varIDs(0), split_select_weights(0), case_weights(
+        0), manual_inbag(0), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), regularization_factor(0), regularization_usedepth(
         false), split_varIDs_used(0), variable_importance(0), importance_mode(DEFAULT_IMPORTANCE_MODE), sample_with_replacement(
         true), sample_fraction(0), memory_saving_splitting(false), splitrule(DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(
         DEFAULT_MINPROP), num_random_splits(DEFAULT_NUM_RANDOM_SPLITS), max_depth(DEFAULT_MAXDEPTH), depth(0), last_left_nodeID(
@@ -27,22 +27,21 @@ Tree::Tree() :
 
 Tree::Tree(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
     std::vector<double>& split_values) :
-    mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), deterministic_varIDs(0), split_select_varIDs(0), split_select_weights(
-        0), case_weights(0), manual_inbag(0), split_varIDs(split_varIDs), split_values(split_values), child_nodeIDs(
-        child_nodeIDs), oob_sampleIDs(0), holdout(false), keep_inbag(false), data(0), regularization_factor(0), regularization_usedepth(false), split_varIDs_used(
+    mtry(0), num_samples(0), num_samples_oob(0), min_node_size(0), min_bucket(0), deterministic_varIDs(0), split_select_weights(0), case_weights(
+        0), manual_inbag(0), split_varIDs(split_varIDs), split_values(split_values), child_nodeIDs(child_nodeIDs), oob_sampleIDs(
+        0), holdout(false), keep_inbag(false), data(0), regularization_factor(0), regularization_usedepth(false), split_varIDs_used(
         0), variable_importance(0), importance_mode(DEFAULT_IMPORTANCE_MODE), sample_with_replacement(true), sample_fraction(
         0), memory_saving_splitting(false), splitrule(DEFAULT_SPLITRULE), alpha(DEFAULT_ALPHA), minprop(
         DEFAULT_MINPROP), num_random_splits(DEFAULT_NUM_RANDOM_SPLITS), max_depth(DEFAULT_MAXDEPTH), depth(0), last_left_nodeID(
         0) {
 }
 
-void Tree::init(const Data* data, uint mtry, size_t num_samples, uint seed,
-    std::vector<size_t>* deterministic_varIDs, std::vector<size_t>* split_select_varIDs,
-    std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
+void Tree::init(const Data* data, uint mtry, size_t num_samples, uint seed, std::vector<size_t>* deterministic_varIDs,
+    std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size, uint min_bucket,
     bool sample_with_replacement, bool memory_saving_splitting, SplitRule splitrule, std::vector<double>* case_weights,
     std::vector<size_t>* manual_inbag, bool keep_inbag, std::vector<double>* sample_fraction, double alpha,
-    double minprop, bool holdout, uint num_random_splits, uint max_depth, 
-    std::vector<double>* regularization_factor, bool regularization_usedepth, std::vector<bool>* split_varIDs_used) {
+    double minprop, bool holdout, uint num_random_splits, uint max_depth, std::vector<double>* regularization_factor,
+    bool regularization_usedepth, std::vector<bool>* split_varIDs_used) {
 
   this->data = data;
   this->mtry = mtry;
@@ -58,10 +57,10 @@ void Tree::init(const Data* data, uint mtry, size_t num_samples, uint seed,
   random_number_generator.seed(seed);
 
   this->deterministic_varIDs = deterministic_varIDs;
-  this->split_select_varIDs = split_select_varIDs;
   this->split_select_weights = split_select_weights;
   this->importance_mode = importance_mode;
   this->min_node_size = min_node_size;
+  this->min_bucket = min_bucket;
   this->sample_with_replacement = sample_with_replacement;
   this->splitrule = splitrule;
   this->case_weights = case_weights;
@@ -91,7 +90,7 @@ void Tree::grow(std::vector<double>* variable_importance) {
 
   this->variable_importance = variable_importance;
 
-// Bootstrap, dependent if weighted or not and with or without replacement
+  // Bootstrap, dependent if weighted or not and with or without replacement
   if (!case_weights->empty()) {
     if (sample_with_replacement) {
       bootstrapWeighted();
@@ -156,7 +155,7 @@ void Tree::predict(const Data* prediction_data, bool oob_prediction) {
 
   prediction_terminal_nodeIDs.resize(num_samples_predict, 0);
 
-// For each sample start in root, drop down the tree and return final value
+  // For each sample start in root, drop down the tree and return final value
   for (size_t i = 0; i < num_samples_predict; ++i) {
     size_t sample_idx;
     if (oob_prediction) {
@@ -208,7 +207,7 @@ void Tree::computePermutationImportance(std::vector<double>& forest_importance,
 
   size_t num_independent_variables = data->getNumCols();
 
-// Compute normal prediction accuracy for each tree. Predictions already computed..
+  // Compute normal prediction accuracy for each tree. Predictions already computed..
   double accuracy_normal;
   std::vector<double> prederr_normal_casewise;
   std::vector<double> prederr_shuf_casewise;
@@ -223,33 +222,46 @@ void Tree::computePermutationImportance(std::vector<double>& forest_importance,
   prediction_terminal_nodeIDs.clear();
   prediction_terminal_nodeIDs.resize(num_samples_oob, 0);
 
-// Reserve space for permutations, initialize with oob_sampleIDs
+  // Reserve space for permutations, initialize with oob_sampleIDs
   std::vector<size_t> permutations(oob_sampleIDs);
 
-// Randomly permute for all independent variables
+  // Randomly permute for all independent variables
   for (size_t i = 0; i < num_independent_variables; ++i) {
 
-    // Permute and compute prediction accuracy again for this permutation and save difference
-    permuteAndPredictOobSamples(i, permutations);
-    double accuracy_permuted;
-    if (importance_mode == IMP_PERM_CASEWISE) {
-      accuracy_permuted = computePredictionAccuracyInternal(&prederr_shuf_casewise);
-      for (size_t j = 0; j < num_samples_oob; ++j) {
-        size_t pos = i * num_samples + oob_sampleIDs[j];
-        forest_importance_casewise[pos] += prederr_shuf_casewise[j] - prederr_normal_casewise[j];
+    // Check whether the i-th variable is used in the
+	  // tree:
+    bool isused = false;
+    for (size_t j = 0; j < split_varIDs.size(); ++j) {
+      if (split_varIDs[j] == i) {
+        isused = true;
+        break;
       }
-    } else {
-      accuracy_permuted = computePredictionAccuracyInternal(NULL);
     }
-
-    double accuracy_difference = accuracy_normal - accuracy_permuted;
-    forest_importance[i] += accuracy_difference;
-
-    // Compute variance
-    if (importance_mode == IMP_PERM_BREIMAN) {
-      forest_variance[i] += accuracy_difference * accuracy_difference;
-    } else if (importance_mode == IMP_PERM_LIAW) {
-      forest_variance[i] += accuracy_difference * accuracy_difference * num_samples_oob;
+     
+	 // Only do permutations if the variable is used in the tree, otherwise variable importance is 0
+    if (isused) {
+      // Permute and compute prediction accuracy again for this permutation and save difference
+      permuteAndPredictOobSamples(i, permutations);
+      double accuracy_permuted;
+      if (importance_mode == IMP_PERM_CASEWISE) {
+        accuracy_permuted = computePredictionAccuracyInternal(&prederr_shuf_casewise);
+        for (size_t j = 0; j < num_samples_oob; ++j) {
+          size_t pos = i * num_samples + oob_sampleIDs[j];
+          forest_importance_casewise[pos] += prederr_shuf_casewise[j] - prederr_normal_casewise[j];
+        }
+      } else {
+        accuracy_permuted = computePredictionAccuracyInternal(NULL);
+      }
+  
+      double accuracy_difference = accuracy_normal - accuracy_permuted;
+      forest_importance[i] += accuracy_difference;
+  
+      // Compute variance
+      if (importance_mode == IMP_PERM_BREIMAN) {
+        forest_variance[i] += accuracy_difference * accuracy_difference;
+      } else if (importance_mode == IMP_PERM_LIAW) {
+        forest_variance[i] += accuracy_difference * accuracy_difference * num_samples_oob;
+      }
     }
   }
 }
@@ -257,12 +269,12 @@ void Tree::computePermutationImportance(std::vector<double>& forest_importance,
 // #nocov start
 void Tree::appendToFile(std::ofstream& file) {
 
-// Save general fields
+  // Save general fields
   saveVector2D(child_nodeIDs, file);
   saveVector1D(split_varIDs, file);
   saveVector1D(split_values, file);
 
-// Call special functions for subclasses to save special fields.
+  // Call special functions for subclasses to save special fields.
   appendToFileInternal(file);
 }
 // #nocov end
@@ -284,7 +296,7 @@ void Tree::createPossibleSplitVarSubset(std::vector<size_t>& result) {
       drawWithoutReplacementSkip(result, random_number_generator, num_vars, (*deterministic_varIDs), mtry);
     }
   } else {
-    drawWithoutReplacementWeighted(result, random_number_generator, *split_select_varIDs, mtry, *split_select_weights);
+    drawWithoutReplacementWeighted(result, random_number_generator, num_vars, mtry, *split_select_weights);
   }
 
   // Always use deterministic variables
@@ -378,7 +390,7 @@ void Tree::createEmptyNode() {
 
 size_t Tree::dropDownSamplePermuted(size_t permuted_varID, size_t sampleID, size_t permuted_sampleID) {
 
-// Start in root and drop down
+  // Start in root and drop down
   size_t nodeID = 0;
   while (child_nodeIDs[0][nodeID] != 0 || child_nodeIDs[1][nodeID] != 0) {
 
@@ -419,11 +431,11 @@ size_t Tree::dropDownSamplePermuted(size_t permuted_varID, size_t sampleID, size
 
 void Tree::permuteAndPredictOobSamples(size_t permuted_varID, std::vector<size_t>& permutations) {
 
-// Permute OOB sample
-//std::vector<size_t> permutations(oob_sampleIDs);
+  // Permute OOB sample
+  //std::vector<size_t> permutations(oob_sampleIDs);
   std::shuffle(permutations.begin(), permutations.end(), random_number_generator);
 
-// For each sample, drop down the tree and add prediction
+  // For each sample, drop down the tree and add prediction
   for (size_t i = 0; i < num_samples_oob; ++i) {
     size_t nodeID = dropDownSamplePermuted(permuted_varID, oob_sampleIDs[i], permutations[i]);
     prediction_terminal_nodeIDs[i] = nodeID;
@@ -432,26 +444,26 @@ void Tree::permuteAndPredictOobSamples(size_t permuted_varID, std::vector<size_t
 
 void Tree::bootstrap() {
 
-// Use fraction (default 63.21%) of the samples
+  // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
 
-// Reserve space, reserve a little more to be save)
+  // Reserve space, reserve a little more to be save)
   sampleIDs.reserve(num_samples_inbag);
   oob_sampleIDs.reserve(num_samples * (exp(-(*sample_fraction)[0]) + 0.1));
 
   std::uniform_int_distribution<size_t> unif_dist(0, num_samples - 1);
 
-// Start with all samples OOB
+  // Start with all samples OOB
   inbag_counts.resize(num_samples, 0);
 
-// Draw num_samples samples with replacement (num_samples_inbag out of n) as inbag and mark as not OOB
+  // Draw num_samples samples with replacement (num_samples_inbag out of n) as inbag and mark as not OOB
   for (size_t s = 0; s < num_samples_inbag; ++s) {
     size_t draw = unif_dist(random_number_generator);
     sampleIDs.push_back(draw);
     ++inbag_counts[draw];
   }
 
-// Save OOB samples
+  // Save OOB samples
   for (size_t s = 0; s < inbag_counts.size(); ++s) {
     if (inbag_counts[s] == 0) {
       oob_sampleIDs.push_back(s);
@@ -467,19 +479,19 @@ void Tree::bootstrap() {
 
 void Tree::bootstrapWeighted() {
 
-// Use fraction (default 63.21%) of the samples
+  // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
 
-// Reserve space, reserve a little more to be save)
+  // Reserve space, reserve a little more to be save)
   sampleIDs.reserve(num_samples_inbag);
   oob_sampleIDs.reserve(num_samples * (exp(-(*sample_fraction)[0]) + 0.1));
 
   std::discrete_distribution<> weighted_dist(case_weights->begin(), case_weights->end());
 
-// Start with all samples OOB
+  // Start with all samples OOB
   inbag_counts.resize(num_samples, 0);
 
-// Draw num_samples samples with replacement (n out of n) as inbag and mark as not OOB
+  // Draw num_samples samples with replacement (n out of n) as inbag and mark as not OOB
   for (size_t s = 0; s < num_samples_inbag; ++s) {
     size_t draw = weighted_dist(random_number_generator);
     sampleIDs.push_back(draw);
@@ -510,7 +522,7 @@ void Tree::bootstrapWeighted() {
 
 void Tree::bootstrapWithoutReplacement() {
 
-// Use fraction (default 63.21%) of the samples
+  // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
   shuffleAndSplit(sampleIDs, oob_sampleIDs, num_samples, num_samples_inbag, random_number_generator);
   num_samples_oob = oob_sampleIDs.size();
@@ -526,17 +538,17 @@ void Tree::bootstrapWithoutReplacement() {
 
 void Tree::bootstrapWithoutReplacementWeighted() {
 
-// Use fraction (default 63.21%) of the samples
+  // Use fraction (default 63.21%) of the samples
   size_t num_samples_inbag = (size_t) num_samples * (*sample_fraction)[0];
   drawWithoutReplacementWeighted(sampleIDs, random_number_generator, num_samples - 1, num_samples_inbag, *case_weights);
 
-// All observation are 0 or 1 times inbag
+  // All observation are 0 or 1 times inbag
   inbag_counts.resize(num_samples, 0);
   for (auto& sampleID : sampleIDs) {
     inbag_counts[sampleID] = 1;
   }
 
-// Save OOB samples. In holdout mode these are the cases with 0 weight.
+  // Save OOB samples. In holdout mode these are the cases with 0 weight.
   if (holdout) {
     for (size_t s = 0; s < (*case_weights).size(); ++s) {
       if ((*case_weights)[s] == 0) {
